@@ -1,4 +1,4 @@
-/* Copyright 2007-2010 Jozsef Kadlecsik (kadlec@blackhole.kfki.hu)
+/* Copyright 2007-2010 Jozsef Kadlecsik (kadlec@netfilter.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -62,6 +62,13 @@ enum ipset_opt {
 	IPSET_OPT_BYTES,
 	IPSET_OPT_CREATE_COMMENT,
 	IPSET_OPT_ADT_COMMENT,
+	IPSET_OPT_SKBINFO,
+	IPSET_OPT_SKBMARK,
+	IPSET_OPT_SKBPRIO,
+	IPSET_OPT_SKBQUEUE,
+	IPSET_OPT_IFACE_WILDCARD,
+	IPSET_OPT_BUCKETSIZE,
+	IPSET_OPT_INITVAL,
 	/* Internal options */
 	IPSET_OPT_FLAGS = 48,	/* IPSET_FLAG_EXIST| */
 	IPSET_OPT_CADT_FLAGS,	/* IPSET_FLAG_BEFORE| */
@@ -70,6 +77,7 @@ enum ipset_opt {
 	IPSET_OPT_LINENO,
 	IPSET_OPT_REVISION,
 	IPSET_OPT_REVISION_MIN,
+	IPSET_OPT_INDEX,
 	IPSET_OPT_MAX,
 };
 
@@ -96,7 +104,8 @@ enum ipset_opt {
 	| IPSET_FLAG(IPSET_OPT_SIZE)	\
 	| IPSET_FLAG(IPSET_OPT_COUNTERS)\
 	| IPSET_FLAG(IPSET_OPT_CREATE_COMMENT)\
-	| IPSET_FLAG(IPSET_OPT_FORCEADD))
+	| IPSET_FLAG(IPSET_OPT_FORCEADD)\
+	| IPSET_FLAG(IPSET_OPT_SKBINFO))
 
 #define IPSET_ADT_FLAGS			\
 	(IPSET_FLAG(IPSET_OPT_IP)	\
@@ -119,10 +128,18 @@ enum ipset_opt {
 	| IPSET_FLAG(IPSET_OPT_NOMATCH) \
 	| IPSET_FLAG(IPSET_OPT_PACKETS)	\
 	| IPSET_FLAG(IPSET_OPT_BYTES)	\
-	| IPSET_FLAG(IPSET_OPT_ADT_COMMENT))
+	| IPSET_FLAG(IPSET_OPT_ADT_COMMENT)\
+	| IPSET_FLAG(IPSET_OPT_SKBMARK)	\
+	| IPSET_FLAG(IPSET_OPT_SKBPRIO)	\
+	| IPSET_FLAG(IPSET_OPT_SKBQUEUE) \
+	| IPSET_FLAG(IPSET_OPT_IFACE_WILDCARD))
 
 struct ipset_data;
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 extern void ipset_strlcpy(char *dst, const char *src, size_t len);
 extern void ipset_strlcat(char *dst, const char *src, size_t len);
 extern bool ipset_data_flags_test(const struct ipset_data *data,
@@ -156,4 +173,8 @@ extern void ipset_data_fini(struct ipset_data *data);
 
 extern size_t ipset_data_sizeof(enum ipset_opt opt, uint8_t family);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* LIBIPSET_DATA_H */
