@@ -418,9 +418,9 @@ static const struct ipset_attr_policy create_attrs[] = {
 		.type = MNL_TYPE_U32,
 		.opt = IPSET_OPT_CADT_FLAGS,
 	},
-	[IPSET_ATTR_GC] = {
+	[IPSET_ATTR_INITVAL] = {
 		.type = MNL_TYPE_U32,
-		.opt = IPSET_OPT_GC,
+		.opt = IPSET_OPT_INITVAL,
 	},
 	[IPSET_ATTR_HASHSIZE] = {
 		.type = MNL_TYPE_U32,
@@ -438,9 +438,9 @@ static const struct ipset_attr_policy create_attrs[] = {
 		.type = MNL_TYPE_U8,
 		.opt = IPSET_OPT_NETMASK,
 	},
-	[IPSET_ATTR_PROBES] = {
+	[IPSET_ATTR_BUCKETSIZE] = {
 		.type = MNL_TYPE_U8,
-		.opt = IPSET_OPT_PROBES,
+		.opt = IPSET_OPT_BUCKETSIZE,
 	},
 	[IPSET_ATTR_RESIZE] = {
 		.type = MNL_TYPE_U8,
